;CPUID for 8080/8085/z80 and compatibles
;Version 1.51
;Ivan Gorodetsky 20.01.2014
;
;Usage:
;CPUID151	- without interrupt usage
;CPUID151 /I	- with interrupt usage for detection of CMOS/NMOS Z80 or NEC emulation mode
;
;Used some code and ideas from
; Ermolaev Sergey (SES)
; Perlin Vitaly (PPC)
; Roshin Ivan
; Slavinsky Vjacheslav (svofski)

#define LD_A_R .db 0EDh,05Fh


		.org 100h


		lxi	h, 80h
		xra	a
		ora	m
		jz	SkipI
		inx	h
		inx	h
		mvi	a,'/'
		cmp	m
		jnz	SkipI
		inx	h
		mov	a, m
		cpi	'I'
		jz	UseInt
		cpi	'i'
		jnz	SkipI
UseInt:
		xra	a	;nop
		sta	z80stopper
		mvi	a, 0F3h	;di
		sta	SetDI
		mvi	a, 0FBh	;ei
		sta	SetEI1
		sta	SetEI2
SkipI:
		lxi 	d, msg_cpu
		mvi 	c, 9
		call 	5
		call 	cpudetect
		mvi	c, 9
		call 	5
		ret
msg_cpu		.db	"CPU: $"
msg_z80		.db	"Z80$"
msg_cmosz80	.db	"CMOS Z80$"
msg_nmosz80	.db	"NMOS Z80$"
msg_r800	.db	"R800$"
msg_i8080	.db	"I8080$"
msg_amd8080	.db	"AMD8080$"
msg_necv	.db	"NEC VX0 (EMULATION MODE)$"
msg_8085	.db	"8085$"
msg_vm1		.db	"580VM1$"
cpudetect:	sub	a
		jpo	difz80
		push	psw
		pop	h
		mov	e, l
		mvi	a, 00100010b
		xra	l
		mov	l, a
		push	h
		pop	psw
		push	psw
		pop	h
		mov	a,l
		xra	e
		jz	dif8080
		lxi	d, msg_8085
		rpe
		lxi	d, msg_vm1
		ret
dif8080:
		call	CheckNEC
SetEI1:		nop
		ani	00001000b
		daa
		lxi	d, msg_amd8080
		rz
		lxi	d, msg_i8080
		ret
CheckNEC:
		mvi	e, 0C9h	;ret
		lxi	h,NECnotRET
SetDI:		nop\ nop\ nop\ nop
		nop\ nop\ nop\ nop
		nop\ nop\ nop\ nop
		nop\ nop\ nop\ nop
		mov	m, e
NECnotRET:	nop
		pop	h
		lxi	d, msg_necv
SetEI2:		nop
		ret

difz80:
		mvi	e,1
		mov	l,e
		.db 0EDh,0D9h	;mulub a,e for r800 / nop for z80
		ora	l
		lxi	d, msg_r800
		rz
		lxi	d, msg_z80
z80stopper:	ret
		LD_A_R
		jpe	m1
		LD_A_R
m1:		push	psw
		ei
		lxi	b,0
z80loop:
		LD_A_R
		jpo	nmosz80
		LD_A_R
		jpo	nmosz80
		LD_A_R
		jpo	nmosz80
		LD_A_R
		jpo	nmosz80
		inr	c
		jnz	z80loop
		inr	b
		jnz	z80loop
		lxi	d,msg_cmosz80
		pop	psw
		di
		rpo
		ei
		ret
nmosz80:	lxi	d,msg_nmosz80
		pop	psw
		di
		rpo
		ei
		ret



		.end